# Copyright (C) 2017  Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; witout even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import botogram


class AuthComponent(botogram.Component):
    """Check if the user is authorized to contact the bot"""
    component_name = "auth"

    def __init__(self, db):
        self.db = db

        self.add_before_processing_hook(self.check)
        self.add_command("authorize", self.authorize_command)

    def check(self, chat):
        """Check if a chat is authorized to talk with the bot"""
        everyone = [r[0] for r in self.db.query("SELECT id FROM auth;")]

        # If this is the first chat to talk with the bot, authorize it
        if not everyone:
            self.db.update("INSERT INTO auth (id) VALUES (?);", chat.id)
        else:
            # Don't do anything if the user is authorized
            if chat.id in everyone:
                return

            chat.send("\n".join((
                "‼️ *Non sei autorizzato a contattare questo bot!*",
                "Chiedi all'amministratore di digitare "
                "`/authorize %s`" % chat.id
            )))

            if chat.type in ("group", "supergroup"):
                chat.leave()

            return True

    def authorize_command(self, message, args):
        """Autorizza un ID a contattare il bot"""
        if len(args) != 1:
            message.reply("‼️ *Uso:* `/authorize <id>`")
            return

        if self.db.query("SELECT id FROM auth WHERE id = ?;", args[0]):
            message.reply("‼️ *Errore:* Chat già autorizzata!")
            return

        self.db.update("INSERT INTO auth (id) VALUES (?);", args[0])
        message.reply("✅ La chat è ora *autorizzata* a contattare il bot.")
