# Copyright (C) 2017  Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; witout even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json

import botogram
import botogram.objects.base

from .db import Database
from .auth import AuthComponent
from .buffer import Buffer
from .settings import SettingsComponent
from .post import PostComponent


class CallbackQuery(botogram.objects.base.BaseObject):
    required = {
        "id": str,
        "from": botogram.User,
        "chat_instance": str,
    }
    optional = {
        "inline_message_id": str,
        "message": botogram.Message,
        "data": str,
        "game_short_name": str,
    }
    replace_keys = {
        "from": "sender",
    }

botogram.Update.optional["callback_query"] = CallbackQuery


def create_bot(token, db_path):
    """Create a new instance of the bot"""
    bot = botogram.create(token)
    bot.lang = "it"

    db = Database(db_path)
    db.init()

    buffer = Buffer(db)

    auth = AuthComponent(db)
    settings = SettingsComponent(db, buffer)
    post = PostComponent(db, buffer, auth)

    bot.use(auth)
    bot.use(settings)
    bot.use(post)

    bot.register_update_processor("callback_query", post.process_callbacks)

    return bot
