# Copyright (C) 2017  Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; witout even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import requests
import botogram


class InvalidBufferTokenError(Exception):
    pass


class Buffer:
    """Interface with Buffer"""

    def __init__(self, db, base=None):
        if base is None:
            base = "https://api.bufferapp.com/1"
        self.base = base

        self.db = db

    def set_access_token(self, token):
        """Set the new access token"""
        # Check if the token works
        self.request("get", "profiles.json", {
            "access_token": token,
        })

        # If the previous method didn't trigger an exception...
        self.db.update(
                "INSERT OR REPLACE INTO kw VALUES ('buffer_access_token', ?);",
            token
        )

    def access_token(self):
        """Get the current access token"""
        data = self.db.query(
            "SELECT value FROM kw WHERE key = 'buffer_access_token';"
        )

        if not data:
            raise InvalidBufferTokenError
        return data[0]

    def request(self, method, url, params=None, data=None):
        """Make a new request to Buffer"""
        if params is None:
            params = {}
        if "access_token" not in params:
            params["access_token"] = self.access_token()

        res = requests.request(method,
            "%s/%s" % (self.base, url), params=params, data=data
        )
        if res.status_code == 401:
            raise InvalidBufferTokenError

        return res.json()
