# Copyright (C) 2017  Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; witout even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import threading
import sqlite3

import pkg_resources


class Database:

    def __init__(self, path):
        self._path = path

    def init(self):
        """Initialize the database"""
        # Get the path to the schema file
        schema = pkg_resources.resource_filename("uitsmmbot", "schema.sql")

        # Migrate the database
        cursor = self.cursor()
        with open(schema) as f:
            cursor.executescript(f.read())

    def cursor(self):
        """Get a new cursor"""
        local = threading.local()

        if not hasattr(local, "db"):
            local.db = sqlite3.connect(self._path)
        return local.db.cursor()

    def query(self, query, *params, update=False):
        """Make a new query against the db"""
        cursor = self.cursor()
        cursor.execute(query, params)

        try:
            if update:
                cursor.connection.commit()
            else:
                return cursor.fetchall()
        finally:
            cursor.close()

    def update(self, query, *params):
        """Make a new update query against the db"""
        self.query(query, *params, update=True)
