# Copyright (C) 2017  Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; witout even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import botogram

from .buffer import InvalidBufferTokenError


class SettingsComponent(botogram.Component):
    component_name = "buffer_settings"

    def __init__(self, db, buffer):
        self.db = db
        self.buffer = buffer

        self.add_command("buffer_access_token", self.buffer_access_token)
        self.add_command("link_telegram", self.link_telegram)
        self.add_command("unlink_telegram", self.unlink_telegram)

    def buffer_access_token(self, message, args):
        """Imposta un nuovo token di Buffer"""
        if len(args) != 1:
            message.reply("‼️ *Uso:* `/buffer_access_token <nuovo-token>`")
            return

        try:
            self.buffer.set_access_token(args[0])
        except InvalidBufferTokenError:
            message.reply("‼️ Il token inserito *non è valido*!")
        else:
            message.reply("✅ Il token di Buffer è stato *aggiornato*!")

    def link_telegram(self, bot, message, args):
        """Collega un canale telegram al bot"""
        if len(args) != 1:
            message.reply("‼️ *Uso:* `/link_telegram <username-canale>`")
            return

        if self.db.query("SELECT * FROM kw WHERE key = 'telegram_channel';"):
            message.reply("‼️ *Errore:* c'è già un canale collegato.")
            return

        try:
            status = bot.api.call("getChatMember", {
                "chat_id": args[0],
                "user_id": bot.itself.id,
            })["result"]["status"]
        except botogram.APIError:
            message.reply("‼️ *Errore:* il canale non esiste!")
            return

        if status != "administrator":
            message.reply("‼️ *Errore:* il bot non è amministratore del canale!")
            return

        self.db.update(
            "INSERT INTO kw (key, value) VALUES ('telegram_channel', ?);",
            args[0],
        )
        message.reply("✅ Canale *collegato* con successo!")

    def unlink_telegram(self, message):
        """Rimuovi il collegamento dal canale telegram"""
        self.db.update("DELETE FROM kw WHERE key = 'telegram_channel';")
        message.reply("✅ Link al canale Telegram *rimosso*.")
